import logging
import os
import sys
import time

import httpx

from canopto.classes.menuoption import MenuOption
from canopto.core.config import select_download_dir_dialog
from canopto.ui.messages import print_download_time, print_request_error, print_faq
from canopto.ui.tui import select_option_prompt


async def download_files(courses):
    start = time.perf_counter()
    await courses.download_files()
    end = time.perf_counter() - start
    time_taken = f'{end:0.2f}'
    print_download_time("files", time_taken)


async def download_videos(courses):
    start = time.perf_counter()
    for course in courses.courses:
        await course.get_videos()
    await course.download_videos()
    end = time.perf_counter() - start
    time_taken = f'{end:0.2f}'
    print_download_time("videos", time_taken)


async def download_files_and_videos(courses):
    await download_files(courses)
    await download_videos(courses)


def show_faq():
    print_faq()


def clear_screen():
    if sys.platform == "win32":
        os.system('cls')
    else:
        os.system('clear')


async def select_menu(courses):
    clear_screen()

    while True:
        option = await select_option_prompt(courses)
        clear_screen()

        try:
            if option == MenuOption.DL_ALL:
                await download_files_and_videos(courses)

            elif option == MenuOption.DL_FILES:
                await download_files(courses)

            elif option == MenuOption.DL_VIDEOS:
                await download_videos(courses)

            elif option == MenuOption.SEL_FOLDER:
                await select_download_dir_dialog()

            elif option == MenuOption.FAQ:
                show_faq()

            elif option == MenuOption.QUIT:
                break

        except httpx.RequestError as exc:
            print_request_error()
            logging.error(f"Menu: An error occurred while requesting {exc.request.url!r}.")
            break



from prompt_toolkit import print_formatted_text, HTML
from prompt_toolkit.formatted_text import FormattedText
from prompt_toolkit.styles import Style

from canopto.core.config import get_download_dir, is_download_dir_valid

STYLE = Style.from_dict({
    'error': 'fg:#ff0066',
    'warning': 'fg:ansiyellow',
    'info': 'fg:ansibrightblue',
    'time': 'fg:orange',
    'underline': 'underline',
    'ub': 'underline bold',
    'cyan': 'fg: ansicyan',
    'strike': 'strike',
})


def print_request_error():
    msg = FormattedText([
        ('class:error', 'Connection Error. Please check your internet connection!')
    ])
    print_formatted_text(msg, style=STYLE)


def print_authentication_error(status_code):
    msg = FormattedText([
        ('class:error', f'Error {status_code}. Please check your username and password!')
    ])
    print_formatted_text(msg, style=STYLE)


def print_folder_warning():
    msg = FormattedText([
        ('class:warning', 'You have not chosen a folder to store your course files and videos.')
    ])
    print_formatted_text(msg, style=STYLE)
    print()


async def print_folder_notification():
    if await is_download_dir_valid():
        msg = FormattedText([
            ('', 'Folder to store files and videos: '),
            ('class:ansicyan', f'{get_download_dir()}'),
        ])
    else:
        msg = FormattedText([
            ('class:warning', 'You cannot download any files or videos until you choose a folder to store them.')
        ])
    print_formatted_text(msg, style=STYLE)
    print_formatted_text("")


def print_download_time(file_or_video, time_taken):
    msg = FormattedText([
        ('class:time', f'Finished downloading {file_or_video} in {time_taken}s.')
    ])
    print_formatted_text(msg, style=STYLE)


def print_download_time(file_or_video, time_taken):
    msg = FormattedText([
        ('class:time', f'Finished downloading {file_or_video} in {time_taken}s.\n')
    ])
    print_formatted_text(msg, style=STYLE)


def print_list_courses():
    msg = FormattedText([
        ('class:ub', 'You are taking the following courses:')
    ])
    print_formatted_text(msg, style=STYLE)


def print_faq():
    faq_text = HTML("""
<aaa bg='seagreen'><bold>FAQ</bold></aaa>
<underline>1. How are my login credentials stored?</underline>
Your username and password is stored locally and securely on your computer using the python keyring library.
For more information please check out: https://pypi.org/project/keyring/
Credentials are only used for authenticating with NUS Canvas servers.

<underline>2. Video downloads seem to take very long!</underline>
Videos can be large, and can easily be a few hundred MBs each. Please wait patiently.

<underline>3. If my download is interrupted, can I resume it?</underline>
Yes, you can end downloads abruptly and resume them at any time.
Partially downloaded files have a ".part" file extension, which will be removed
once the download is fully complete.

<underline>4. Will my annotated files be overwritten?</underline>
No, if the file has been full downloaded to your computer (no more ".part" extension), it will be ignored.

""")
    print_formatted_text(faq_text)



from pathlib import Path

import aiofiles
import aiofiles.os
import httpx

API_BASE_URL = 'https://canvas.nus.edu.sg/api/v1/'
BROWSER_HEADER = {
    'User-Agent': 'Mozilla/5.0 (Windows NT 10.0; rv:104.0) Gecko/20100101 Firefox/104.0',
    'Accept': 'text/html,application/xhtml+xml,application/xml;q=0.9,image/avif,image/clientp,*/*;q=0.8',
    'Accept-Language': 'en-US,en;q=0.5',
    'Accept-Encoding': 'gzip, deflate',
    'Connection': 'keep-alive',
    'Upgrade-Insecure-Requests': '1',
    'Sec-Fetch-Dest': 'document',
    'Sec-Fetch-Mode': 'navigate',
    'Sec-Fetch-Site': 'none',
    'Sec-Fetch-User': '?1',
    'TE': 'trailers'
}

LIMITS = httpx.Limits(max_keepalive_connections=5, max_connections=10)
ACLIENT = httpx.AsyncClient(http1=False, http2=True,
                            limits=LIMITS,
                            headers=BROWSER_HEADER, timeout=3)


async def close_aclient():
    await ACLIENT.aclose()


async def get_request(url, follow_redirects=False):
    res = await ACLIENT.get(url, follow_redirects=follow_redirects)
    if res.status_code >= 400:
        res.raise_for_status()
    return res


async def post_request(url, data=None, json=None, follow_redirects=False):
    res = await ACLIENT.post(url, data=data, json=json,
                             follow_redirects=follow_redirects)
    if res.status_code >= 400:
        res.raise_for_status()
    return res


async def get_json(api_endpoint: str):
    url = API_BASE_URL + api_endpoint
    response = await get_request(url)
    return response.json()


async def stream_download(headers, url, file_path, file_path_part):
    async with aiofiles.open(file_path_part, mode='ab') as file:
        print(f"Downloading {file_path} ...")
        async with ACLIENT.stream('GET', url, headers=headers, follow_redirects=True) as res:
            res.raise_for_status()
            async for chunk in res.aiter_bytes():
                await file.write(chunk)
        print(f"Finished downloading {file_path}!")


async def download_file(url, path):
    """
Partially downloaded files are created with .part extension.
Even if download stops halfway, it can be resumed from where it left off.
Once fully downloaded, .part extension is removed.
Files stored locally without .part are ignored.
    @param url: url of file to be downloaded
    @param path: absolute path of file to be downloaded
    """
    file_path = Path(path)
    file_path_part = Path(f'{path}.part')

    if file_path_part.exists():
        file_size = file_path_part.stat().st_size
        headers = BROWSER_HEADER
        headers.update({'range': f'bytes={file_size}-'})
        await stream_download(headers, url, file_path, file_path_part)
        await aiofiles.os.rename(file_path_part, file_path)

    else:
        if not file_path.exists():
            await stream_download(BROWSER_HEADER, url, file_path, file_path_part)
            await aiofiles.os.rename(file_path_part, file_path)


def ep_personal_folders() -> str:
    return "users/self/folders"


def ep_courses() -> str:
    return "courses"


def ep_folders_in_course(course_id: int) -> str:
    return f'courses/{course_id}/folders/'


def ep_files_in_folder(folder_id: int) -> str:
    return f'folders/{folder_id}/files/'


def get_ext_tools_panopto_url(course_id):
    return f'https://canvas.nus.edu.sg/courses/{course_id}/external_tools/128'


# Can use this to arbitrarily search panopto videos
def create_folder_request_payload(folderId):
    return {
        "queryParameters": {
            # "bookmarked": False,
            # "endDate": None,
            "folderID": folderId,
            "getFolderData": True,
            "includeArchived": True,
            "includeArchivedStateCount": True,
            "includePlaylists": True,
            # "isSharedWithMe": False,
            # "isSubscriptionsPage": False,
            # "maxResults": 25,
            # "page": 0,
            # "query": None,
            "sessionListOnlyArchived": False,
            "sortAscending": True,
            # "sortColumn": 1,
            # "startDate": None
        }
    }



import asyncio
import logging
import os

import aiofiles

from canopto.classes.file import File
from canopto.core.config import get_download_dir
from canopto.core.web import ep_files_in_folder, get_json


class Folder:

    def __init__(self, m_id: int, path: str, course_code: str):
        self.id = m_id
        self.path = path.replace('course files', course_code)  # replace default folder name given by canvas
        self.files = []

    async def download_files(self) -> None:
        await asyncio.gather(*[file.download() for file in self.files])

    def exists(self) -> bool:
        return os.path.exists(self.path)

    async def refresh(self) -> None:
        files_json = await get_json(ep_files_in_folder(self.id))
        self.files = [
            File(f["id"], f["filename"],
                 f["size"], f["url"], self.path)
            for f in files_json]

    def abs_path(self) -> str:
        return os.path.join(get_download_dir(), self.path)

    async def create_dir(self) -> None:
        logging.info(self.abs_path())
        await aiofiles.os.makedirs(self.abs_path(), exist_ok=True)



from __future__ import unicode_literals

from prompt_toolkit import PromptSession
from prompt_toolkit import print_formatted_text, HTML
from prompt_toolkit.key_binding.key_bindings import KeyBindings
from prompt_toolkit.patch_stdout import patch_stdout
from prompt_toolkit.shortcuts import create_confirm_session
from prompt_toolkit.validation import Validator, ValidationError

from canopto.classes.menuoption import MenuOption
from canopto.core.config import is_download_dir_valid
from canopto.ui.messages import print_folder_notification


class BlankValidatorNoMessage(Validator):
    def validate(self, document):
        text = document.text

        if len(text) == 0:
            raise ValidationError()


class BlankValidatorWithMessage(Validator):
    def validate(self, document):
        text = document.text

        if len(text) == 0:
            raise ValidationError(message='Field cannot be blank!', cursor_position=0)


class OptionValidator(BlankValidatorNoMessage):
    def validate(self, document):
        super().validate(document)
        text = document.text

        if len(text) > 1 or not text.isdigit() or int(text) < 1 or int(text) > 5:
            raise ValidationError(message='Only numbers from 1-5 are allowed', cursor_position=0)


async def get_credentials_prompt():
    session = PromptSession()
    with patch_stdout():
        username = await session.prompt_async('Enter NUSNET ID (exxxxxxx): ', validator=BlankValidatorWithMessage())
        password = await session.prompt_async('Password: ', is_password=True, validator=BlankValidatorWithMessage())
    return username, password


async def ask_save_credentials_prompt():
    session = create_confirm_session("Do you want to save username and password on your computer?",
                                     suffix=" Press (y)es/(n)o: ")
    return await session.prompt_async(validator=BlankValidatorNoMessage())


bindings = KeyBindings()


@bindings.add(str(1))
def exit_with_value(event):
    event.app.exit(result=MenuOption.DL_ALL)


@bindings.add(str(2))
def exit_with_value(event):
    event.app.exit(result=MenuOption.DL_FILES)


@bindings.add(str(3))
def exit_with_value(event):
    event.app.exit(result=MenuOption.DL_VIDEOS)


@bindings.add(str(4))
def exit_with_value(event):
    event.app.exit(result=MenuOption.SEL_FOLDER)


@bindings.add(str(5))
def exit_with_value(event):
    event.app.exit(result=MenuOption.FAQ)


@bindings.add(str(6))
def exit_with_value(event):
    event.app.exit(result=MenuOption.QUIT)


half_bindings = KeyBindings()


@half_bindings.add(str(4))
def exit_with_value(event):
    event.app.exit(result=MenuOption.SEL_FOLDER)


@half_bindings.add(str(5))
def exit_with_value(event):
    event.app.exit(result=MenuOption.FAQ)


@half_bindings.add(str(6))
def exit_with_value(event):
    event.app.exit(result=MenuOption.QUIT)


async def select_option_prompt(courses):
    with patch_stdout():
        courses.list_courses()
        await print_folder_notification()
        print_formatted_text(HTML("<aaa bg='ansiblue' ><b>Select option number:</b></aaa>"))

        if await is_download_dir_valid():
            session = PromptSession(key_bindings=bindings, validator=OptionValidator())
            print_formatted_text(HTML("<aaa >1. Download all files + videos</aaa>"))
            print_formatted_text(HTML("<aaa >2. Download all files </aaa>"))
            print_formatted_text(HTML("<aaa >3. Download all videos</aaa>"))
        else:
            session = PromptSession(key_bindings=half_bindings, validator=OptionValidator())
            print_formatted_text(
                HTML("<aaa ><strike fg='ansibrightblack'>1. Download all files + videos</strike></aaa>"))
            print_formatted_text(HTML("<aaa ><strike fg='ansibrightblack'>2. Download all files</strike></aaa>"))
            print_formatted_text(HTML("<aaa ><strike fg='ansibrightblack'>3. Download all videos</strike></aaa>"))

        print_formatted_text(HTML("<aaa >4. Choose download folder</aaa>"))
        print_formatted_text(HTML("<aaa >5. FAQ</aaa>"))
        print_formatted_text(HTML("<aaa >6. Quit</aaa>\n"))
        print_formatted_text(HTML("<aaa fg='yellow' >Ctrl-C to exit anytime</aaa>"))
        result = await session.prompt_async(editing_mode=False)
        return result



import logging

import httpx
import keyring
from bs4 import BeautifulSoup

from canopto.core.web import get_request, post_request
from canopto.ui.messages import print_authentication_error, print_request_error
from canopto.ui.tui import ask_save_credentials_prompt, get_credentials_prompt

CANVAS_URL = "https://canvas.nus.edu.sg/login/saml/103"
USER_IDENTIFIER_URL = "https://nus.vmwareidentity.asia/federation//auth/login/input/useridentifier"
AUTH_REQUEST_URL = "https://nus.vmwareidentity.asia/authcontrol/auth/request"
AUTHENTICATE_URL = "https://nus.vmwareidentity.asia/authcontrol/authenticate"
AUTH_RESP_INTERNAL_URL = "https://nus.vmwareidentity.asia/federation/auth/response/internal"
SAML_URL = 'https://canvas.nus.edu.sg/login/saml'


def get_credentials_from_keyring():
    try:
        username = keyring.get_password("canopto", "username")
        password = keyring.get_password("canopto", username)
        return username, password
    except keyring.errors.KeyringLocked:
        print("Cannot get saved credentials!")
        return None, None


def save_credentials_to_keyring(username: str, password: str):
    keyring.set_password("canopto", "username", username)
    keyring.set_password("canopto", username, password)


async def authenticate(username, password) -> None:
    """
    Retrieve authentication cookies from Canvas server.
    Cookies can be saved to a file and loaded to minimize the authentication
    process
    """

    # 1 Canvas Login portal
    res = await get_request(CANVAS_URL, follow_redirects=True)

    # 2 User Identifier
    context_id = BeautifulSoup(res.text, 'html.parser').find(
        'input', {'id': 'contextId'})['value']
    data = {'username': username, 'contextId': context_id}
    res = await post_request(USER_IDENTIFIER_URL, data)

    # 3 Auth Request
    data = {i['name']: i['value']
            for i in BeautifulSoup(res.text, 'html.parser').find_all('input')}
    res = await post_request(AUTH_REQUEST_URL, data)

    # 4 Authenticate with credentials
    context_id = BeautifulSoup(res.text, 'html.parser').find(
        'input', {'id': 'contextId'})['value']
    data = {'userInput': username,
            'password': password, 'contextId': context_id}
    res = await post_request(AUTHENTICATE_URL, data)

    # 5 Internal Authentication
    data = {i['name']: i['value']
            for i in BeautifulSoup(res.text, 'html.parser').find_all('input')}
    res = await post_request(AUTH_RESP_INTERNAL_URL,
                             data, follow_redirects=True)

    # 6 SAML (No need to follow_redirect)
    saml_data = BeautifulSoup(res.text, 'html.parser').form.get_text()
    data = {'SAMLResponse': saml_data}
    res = await post_request(SAML_URL, data)


async def authentication_loop():
    username, password = get_credentials_from_keyring()

    is_keyring_used = False
    if username is not None and password is not None:
        is_keyring_used = True

    while True:
        if username is None or password is None:
            username, password = await get_credentials_prompt()

        try:
            await authenticate(username, password)

        except httpx.RequestError as exc:
            print_request_error()
            logging.error(f"An error occurred while requesting {exc.request.url!r}.")
            return False

        except httpx.HTTPStatusError as exc:
            print_authentication_error(exc.response.status_code)
            logging.error(f"Error {exc.response.status_code} while requesting {exc.request.url!r}.")
            username, password = None, None
            is_keyring_used = False  # time for user to enter credentials again

        except httpx.ConnectTimeout as exc:
            logging.error(f"{exc} Timeout error")
            return False

        else:
            if not is_keyring_used:
                if await ask_save_credentials_prompt():
                    save_credentials_to_keyring(username, password)
                    logging.info(f"Saved credentials to keyring")
            return True



import tkinter as tk
from tkinter import filedialog

import aiofiles
import aiofiles.os

download_dir = ''
CONFIG_FILE_PATH = '.canopto_config.txt'


async def parse_config_file():
    """
    Config file should only have download directory at the moment.
    Might have more variables in future.
    Sets the download_dir based on path given in the config file
    """
    if await aiofiles.os.path.exists(CONFIG_FILE_PATH):
        async with aiofiles.open(CONFIG_FILE_PATH, mode='r') as file:
            path = await file.read()
            if await aiofiles.os.path.isdir(path):
                set_download_dir(path)

    return None


async def write_config_file():
    async with aiofiles.open(CONFIG_FILE_PATH, mode='w') as file:
        await file.write(f'{download_dir}')


async def select_download_dir_dialog():
    root = tk.Tk()
    root.withdraw()
    path = filedialog.askdirectory()
    if path != tuple() and path != '':
        set_download_dir(path)
        await write_config_file()


def get_download_dir():
    return download_dir


def set_download_dir(path: str):
    global download_dir
    download_dir = path


async def is_download_dir_valid():
    return await aiofiles.os.path.isdir(download_dir)



import asyncio

from canopto.classes.course import Course
from canopto.core.web import ep_courses, get_json
from canopto.ui.messages import print_list_courses


class Courses:

    def __init__(self, role: str):
        self.role = role
        self.courses = []

    def list_courses(self):
        print_list_courses()
        for course in self.courses:
            print(course)
        print()

    async def download_files(self) -> None:
        await asyncio.gather(*[course.download_files() for course in self.courses])

    async def refresh(self) -> None:
        courses_json = await get_json(ep_courses())

        self.courses = [Course(c["id"], c["name"], c["course_code"])
                        for c in courses_json]

        await asyncio.gather(*[course.refresh() for course in self.courses])

    async def download_videos(self):
        await asyncio.gather(*[course.download_videos() for course in self.courses])



#!/bin/python3
import asyncio

from canopto.classes.courses import Courses
from canopto.core.authenticate import authentication_loop
from canopto.core.config import parse_config_file
from canopto.core.menu import select_menu
from canopto.core.web import close_aclient


# logging.basicConfig(level=logging.INFO)


async def main():
    await parse_config_file()
    is_authenticated = await authentication_loop()

    if is_authenticated:
        courses = Courses('student')
        await courses.refresh()

        await select_menu(courses)

    await close_aclient()


def cli():
    try:
        asyncio.run(main())
    except (KeyboardInterrupt, RuntimeError):
        print("Exited.")


if __name__ == '__main__':
    try:
        asyncio.run(main())
    except (KeyboardInterrupt, RuntimeError):
        print("Exited.")



import logging
import os

from canopto.core.config import get_download_dir
from canopto.core.web import download_file


class File:

    def __init__(self, m_id: str, file_name: str, size: str, url: str, folder_path: str):
        self.id = int(m_id)
        self.path = os.path.join(folder_path, file_name.replace('/', '-'))
        self.size = int(size)
        self.url = url

    # TODO:https://canvas.nus.edu.sg/files/012345/download?download_frd=1
    # save by exact URL or just have a function that calls the download url
    async def download(self) -> None:
        logging.info(self.url)
        await download_file(self.url, self.abs_path())

    def exists(self) -> bool:
        return os.path.exists(self.path)

    def abs_path(self) -> str:
        return os.path.join(get_download_dir(), self.path)

    def same_size(self) -> bool:
        return os.path.getsize(self.abs_path()) == self.size

    def create_dir(self) -> None:
        os.makedirs(self.abs_path())



from setuptools import setup, find_packages

with open('requirements.txt') as f:
    requirements = f.readlines()

setup(
    name='canopto',
    version='1.0.0',
    author='Atticus T',
    author_email='theresurgence2@proton.me',
    url="https://github.com/theresurgence/canopto",
    description="canopto is a tool to sync course files and videos from the Canvas LMS hosted by the National University of Singapore(NUS).",
    license='GPLv3',
    packages=find_packages(),
    entry_points={
            'console_scripts': [
                'canopto=canopto.__main__:cli'
            ]
    },
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: GNU General Public License v3 or later (GPLv3+)",
        "Operating System :: OS Independent",
    ],
    keywords='canopoto canvas panopto nus',
    install_requires=requirements,
    zip_safe=False
)



import asyncio
import logging
import os

import aiofiles
from bs4 import BeautifulSoup

from canopto.classes.folder import Folder
from canopto.core.config import get_download_dir
from canopto.core.web import ep_folders_in_course, get_json
from canopto.core.web import get_ext_tools_panopto_url, get_request, post_request, create_folder_request_payload, \
    download_file

PANOPTO_LTI_URL = 'https://mediaweb.ap.panopto.com/Panopto/LTI/LTI.aspx'
FOLDER_INFO_URL = 'https://mediaweb.ap.panopto.com/Panopto/Services/Data.svc/GetFolderInfo'
FOLDER_SESSION_URL = 'https://mediaweb.ap.panopto.com/Panopto/Services/Data.svc/GetSessions'


class Course:

    def __init__(self, m_id: int, name: str, code: str):
        self.id = m_id
        self.name = name.replace('/', '-')
        self.code = code
        self.folders = []
        self.videos = {}

    def __repr__(self):
        return f'{self.name}'

    async def download_files(self) -> None:
        await self.create_dirs()
        await asyncio.gather(*[folder.download_files() for folder in self.folders])

    async def refresh(self) -> None:
        folders_json = await get_json(ep_folders_in_course(self.id))
        self.folders = [Folder(f["id"], f["full_name"], self.code)
                        for f in folders_json]

        for i in self.folders:
            logging.info(i.path)

        await asyncio.gather(*[folder.refresh() for folder in self.folders])

    async def create_dirs(self) -> None:
        await asyncio.gather(*[folder.create_dir() for folder in self.folders])

    """
    VIDEO
    """

    @staticmethod
    def get_subfolders(folder_path, json_data):
        if len(json_data['d']['Subfolders']) > 0:
            return {os.path.join(folder_path,
                                 i['Name'].replace('/', '-')):
                        i['ID'] for i in json_data['d']['Subfolders']}
        return {}

    @staticmethod
    def get_video_urls(folder_path, json_data):
        if len(json_data['d']['Results']) > 0:
            return {os.path.join(folder_path,
                                 i['SessionName'].replace('/', '-') + '.mp4'):
                        i['IosVideoUrl'] for i in json_data['d']['Results']}
        return {}

    # Extract folder_id from redirected url
    # folder_id = 'd390e6d6-ded7-4bef-870e-aee000201477'
    @staticmethod
    def get_folderid_from_url(url):
        trunc_url = url[url.find('folderID=') + 9:]
        return trunc_url[:trunc_url.find('&')]

    async def get_videos(self):
        # Access panopto through canvas
        url = get_ext_tools_panopto_url(self.id)
        r = await get_request(url)

        # LTI Post request to get .ASPXAUTH COOKIES to Panopto
        data = {i['name']: i['value']
                for i in BeautifulSoup(r.text, 'html.parser').find_all('input')}
        r = await post_request(PANOPTO_LTI_URL, data=data, follow_redirects=True)

        prev_url = str(r.history[-1].url)
        folder_id = self.get_folderid_from_url(prev_url)
        videos_download_dir = os.path.join(get_download_dir(), self.code, 'Videos')

        await self.get_video_dict(videos_download_dir, folder_id)

    async def get_video_dict(self, folder_path, folder_id):

        # Get current panopto course folder data
        json_data = create_folder_request_payload(folder_id)
        r = await post_request(FOLDER_SESSION_URL, data=None, json=json_data)

        # Get video urls and subfolders from current panopto course folder
        self.videos.update(self.get_video_urls(folder_path, r.json()))
        subfolders = self.get_subfolders(folder_path, r.json())

        for folder_path, folder_id in subfolders.items():
            await self.get_video_dict(folder_path, folder_id)

    async def download_videos(self):
        await self.create_video_dirs()
        await asyncio.gather(*[download_file(video_url, video_path)
                               for video_path, video_url
                               in self.videos.items()])

    async def create_video_dirs(self) -> None:
        for video_path in self.videos:
            await aiofiles.os.makedirs(
                os.path.dirname(video_path), exist_ok=True)
